import { ArrayIteratee, ArrayNarrowingIteratee, Cast, IfCouldBe, KeyNarrowingIteratee, Narrow, Nil, ObjectIteratee, ValueNarrowingIteratee } from '../interfaces';
/** @hidden */
declare type DefiniteValueMatches<T, O> = {
    [K in keyof T]: T[K] extends O ? T[K] : never;
}[keyof T];
/** @hidden */
declare type PossibleValueMatches<T, O> = {
    [K in keyof T]: IfCouldBe<T[K], O, Narrow<T[K], O>>;
}[keyof T];
/** @hidden */
declare type DefiniteKeyMatches<T, O> = {
    [K in keyof T]: Cast<K, string> extends O ? T[K] : never;
}[keyof T];
/** @hidden */
declare type PossibleKeyMatches<T, O> = {
    [K in keyof T]: IfCouldBe<Cast<K, string>, O, T[K]>;
}[keyof T];
/**
 * Iterates over elements of `collection`, returning the first element `predicate` returns truthy for.
 *
 * Contribution to minified bundle size, when it is the only function imported:
 * - Lodash: 14,398 bytes
 * - Micro-dash: 231 bytes
 */
export declare function find<I, O>(array: I[] | Nil, predicate: ArrayNarrowingIteratee<O>, fromIndex?: number): Extract<I, O> | Extract<O, I> | undefined;
export declare function find<T>(array: T[] | Nil, predicate: ArrayIteratee<T, boolean>, fromIndex?: number): T | undefined;
export declare function find<I, T extends NonNullable<I>, O, F extends number | undefined = undefined>(object: I, predicate: ValueNarrowingIteratee<T, O>, fromIndex?: F): PossibleValueMatches<T, O> | (DefiniteValueMatches<T, O> extends never ? undefined : never) | IfCouldBe<I, Nil, undefined> | IfCouldBe<F, number, undefined>;
export declare function find<I, T extends NonNullable<I>, O, F extends number | undefined = undefined>(object: I, predicate: KeyNarrowingIteratee<T, O>, fromIndex?: F): PossibleKeyMatches<T, O> | (DefiniteKeyMatches<T, O> extends never ? undefined : never) | IfCouldBe<I, Nil, undefined> | IfCouldBe<F, number, undefined>;
export declare function find<T>(object: T | Nil, predicate: ObjectIteratee<T, boolean>, fromIndex?: number): T[keyof T] | undefined;
export {};
