/** @hidden */
export declare type Nil = null | undefined;
/** @hidden */
export declare type Primitive = boolean | number | string;
/** @hidden */
export declare type Key = keyof any;
/** @hidden */
export declare type Existent = Primitive | object;
/** @hidden */
export declare type ObjectWith<T> = Record<string, T>;
/** @hidden */
export declare type StringifiedKey<T> = Cast<keyof T, string>;
/** @hidden */
export declare type ArrayIteratee<I, O> = (item: I, index: number) => O;
/** @hidden */
export declare type ArrayNarrowingIteratee<O> = (item: any, index: number) => item is O;
/** @hidden */
export declare type ObjectIteratee<T, O> = (item: T[keyof T], key: StringifiedKey<T>) => O;
/** @hidden */
export declare type ValueNarrowingIteratee<I, O> = (item: any, key: StringifiedKey<I>) => item is O;
/** @hidden */
export declare type KeyNarrowingIteratee<I, O> = (item: I[keyof I], key: any) => key is O;
/** @hidden */
export declare type ValueIteratee<T, O> = (value: T) => O;
/** @hidden */
export declare type Cast<I, O> = Exclude<I, O> extends never ? I : O;
/** @hidden */
export declare type Narrow<I, O> = Extract<I, O> | Extract<O, I>;
/** @hidden */
export declare type IfCouldBe<T1, T2, If, Else = never> = Narrow<T1, T2> extends never ? Else : If;
/** @hidden */
export declare type IfIndexType<T, If, Else = never> = string extends T ? If : number extends T ? If : Else;
/** @hidden */
declare type IndexKeys<T> = {
    [K in keyof T]: IfIndexType<K, K>;
}[keyof T];
/** @hidden */
declare type NonIndexKeys<T> = {
    [K in keyof T]: IfIndexType<K, never, K>;
}[keyof T];
/** @hidden */
export declare type PartialExceptIndexes<T> = {
    [K in IndexKeys<T>]: T[K];
} & {
    [K in NonIndexKeys<T>]?: T[K];
};
/** @hidden */
export declare type Evaluate<T> = T extends infer I ? {
    [K in keyof I]: T[K];
} : never;
/** @hidden */
export declare type Drop1Arg<T extends Function> = T extends (arg1: any, ...rest: infer A) => infer R ? (...rest: A) => R : never;
/** @hidden */
export declare type Drop2Args<T extends Function> = T extends (arg1: any, arg2: any, ...rest: infer A) => infer R ? (...rest: A) => R : never;
/** @hidden */
export declare type Drop3Args<T extends Function> = T extends (arg1: any, arg2: any, arg3: any, ...rest: infer A) => infer R ? (...rest: A) => R : never;
/** @hidden */
export declare type Drop4Args<T extends Function> = T extends (arg1: any, arg2: any, arg3: any, arg4: any, ...rest: infer A) => infer R ? (...rest: A) => R : never;
export {};
