import { Cast, Evaluate, IfCouldBe, IfIndexType, KeyNarrowingIteratee, Narrow, Nil, ObjectIteratee, PartialExceptIndexes, ValueNarrowingIteratee } from '../interfaces';
/** @hidden */
declare type IfDefinitelyIncluded<T, O, If, Else = never> = Exclude<T, O> extends never ? If : Else;
/** @hidden */
declare type IfMaybeIncluded<T, O, If, Else = never> = IfDefinitelyIncluded<T, O, Else, IfCouldBe<T, O, If, Else>>;
/** @hidden */
declare type KeysWithDefinitelyIncludedValues<T, O> = {
    [K in keyof T]: IfDefinitelyIncluded<T[K], O, K>;
}[keyof T];
/** @hidden */
declare type KeysWithMaybeIncludedValues<T, O> = {
    [K in keyof T]: IfMaybeIncluded<T[K], O, K>;
}[keyof T];
/** @hidden */
export declare type DefinitelyIncludedKeys<T, O> = {
    [K in keyof T]: IfIndexType<K, IfCouldBe<O, string, IfCouldBe<K, string, Extract<O, K>, K>>, IfDefinitelyIncluded<Cast<K, string>, O, K>>;
}[keyof T];
/** @hidden */
declare type MaybeIncludedKeys<T, O> = {
    [K in keyof T]: IfIndexType<K, never, IfMaybeIncluded<Cast<K, string>, O, K>>;
}[keyof T];
/**
 * Creates an object composed of the `object` properties `predicate` returns truthy for.
 *
 * Differences from lodash:
 * - does not treat sparse arrays as dense
 *
 * Contribution to minified bundle size, when it is the only function imported:
 * - Lodash: 14,767 bytes
 * - Micro-dash: 328 bytes
 */
export declare function pickBy<T, O>(object: T[] | Nil, predicate: ValueNarrowingIteratee<T[], O>): {
    [index: number]: Narrow<T, O>;
};
export declare function pickBy<T>(object: T[] | Nil, predicate: ObjectIteratee<T, boolean>): {
    [index: number]: T;
};
export declare function pickBy<I, T extends NonNullable<I>, O>(object: I, predicate: ValueNarrowingIteratee<T, O>): Evaluate<({
    [K in KeysWithDefinitelyIncludedValues<T, O>]: Narrow<T[K], O>;
} & {
    [K in KeysWithMaybeIncludedValues<T, O>]?: Narrow<T[K], O>;
}) | IfCouldBe<I, Nil, {}>>;
export declare function pickBy<I, T extends NonNullable<I>, O>(object: I, predicate: KeyNarrowingIteratee<T, O>): Evaluate<({
    [K in DefinitelyIncludedKeys<T, O>]: T[K];
} & {
    [K in MaybeIncludedKeys<T, O>]?: T[K];
}) | IfCouldBe<I, Nil, {}>>;
export declare function pickBy<T>(object: T, predicate: ObjectIteratee<T, boolean>): Evaluate<PartialExceptIndexes<NonNullable<T>>>;
export {};
